#define AVS_LINKAGE_DLLIMPORT
#include "avisynth.h"
#ifdef _WIN32
#include "windows.h"
#endif

AVSValue ScriptMonkey(AVSValue args, void*, IScriptEnvironment* env) {
	char* input = (char*)args[0].AsString();
	int p = 0;
	int parens = 0;
	int quote_run = 0;
	bool in_string = false;
	bool triple_quoted = false;
	bool opening_quotes = false;
	int last_non_whitespace = 0;

#ifdef _WIN32
	OutputDebugStringA("==========================================================================");
	OutputDebugStringA("INPUT:");
	OutputDebugStringA(input);
#endif

	while (input[p]) {
		if (!in_string) {
			if (input[p] == '#') { // delete # comments
				while (input[p] && input[p] != 10) input[p++] = ' ';
				continue;
			}

			if (input[p] == '(') ++parens;
			else if (input[p] == ')') {
				if (last_non_whitespace && input[last_non_whitespace] == ',') { // remove trailing comma before )
					input[last_non_whitespace] = ' ';
					last_non_whitespace = 0;
				}
				--parens;
			} else if ((input[p] == 10 || input[p] == 13) && parens) input[p] = ' '; // remove newlines within parentheses
			else if (input[p] == '"') {
				in_string = true;
				opening_quotes = true;
				quote_run = 1;
			} else if (input[p] > 32) last_non_whitespace = p;
		} else { // in string
			if (input[p] == '"') {
				++quote_run;
				if (quote_run == 3) {
					triple_quoted = true;
					if (opening_quotes) {
						quote_run = 0;
						opening_quotes = false;
					}
				}
			} else { // not a "
				if (((quote_run == 1 || quote_run == 2) && !triple_quoted) || (quote_run >= 3 && triple_quoted)) {
					quote_run = 0;
					in_string = false;
					triple_quoted = false;
					continue; // back to top without advancing to check this character
				} else {
					quote_run = 0;
				}
			}
		}
		++p;
	}

#ifdef _WIN32
	OutputDebugStringA("==========================================================================");
	OutputDebugStringA("OUTPUT:");
	OutputDebugStringA(input);
	OutputDebugStringA("==========================================================================");
#endif

	return input;
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment * env) {
	env->AddFunction("ScriptMonkey", "s", ScriptMonkey, 0);

	return "ScriptMonkey";
}
